aboutsummaryrefslogtreecommitdiff
path: root/src/soc/mediatek/common
diff options
context:
space:
mode:
Diffstat (limited to 'src/soc/mediatek/common')
-rw-r--r--src/soc/mediatek/common/dsi.c38
-rw-r--r--src/soc/mediatek/common/include/soc/dsi_common.h3
2 files changed, 23 insertions, 18 deletions
diff --git a/src/soc/mediatek/common/include/soc/dsi_common.h b/src/soc/mediatek/common/include/soc/dsi_common.h
index 3052689c90..25727b8398 100644
--- a/src/soc/mediatek/common/include/soc/dsi_common.h
+++ b/src/soc/mediatek/common/include/soc/dsi_common.h
@@ -16,6 +16,7 @@
#ifndef SOC_MEDIATEK_DSI_COMMON_H
#define SOC_MEDIATEK_DSI_COMMON_H
+#include <commonlib/helpers.h>
#include <edid.h>
#include <types.h>
#include <soc/addressmap.h>
@@ -358,7 +359,7 @@ struct lcm_init_command {
/* Functions that each SOC should provide. */
void mtk_dsi_reset(void);
-void mtk_dsi_configure_mipi_tx(int data_rate, u32 lanes);
+void mtk_dsi_configure_mipi_tx(u32 data_rate, u32 lanes);
/* Functions as weak no-ops that can be overridden. */
void mtk_dsi_override_phy_timing(struct mtk_phy_timing *timing);
diff --git a/src/soc/mediatek/common/dsi.c b/src/soc/mediatek/common/dsi.c
index 238b1eb47f..9222cb0c4b 100644
--- a/src/soc/mediatek/common/dsi.c
+++ b/src/soc/mediatek/common/dsi.c
@@ -39,29 +39,32 @@ static unsigned int mtk_dsi_get_bits_per_pixel(u32 format)
return 24;
}
-static int mtk_dsi_get_data_rate(u32 bits_per_pixel, u32 lanes,
+static u32 mtk_dsi_get_data_rate(u32 bits_per_pixel, u32 lanes,
const struct edid *edid)
{
/* data_rate = pixel_clock * bits_per_pixel * mipi_ratio / lanes
* Note pixel_clock comes in kHz and returned data_rate is in Mbps.
* mipi_ratio is the clk coefficient to balance the pixel clk in MIPI
* for older platforms which do not have complete implementation in HFP.
* Newer platforms should just set that to 1.0 (100 / 100).
*/
- int data_rate = (u64)edid->mode.pixel_clock * bits_per_pixel *
- MTK_DSI_MIPI_RATIO_NUMERATOR /
- (1000 * lanes * MTK_DSI_MIPI_RATIO_DENOMINATOR);
- printk(BIOS_INFO, "DSI data_rate: %d Mbps\n", data_rate);
-
- if (data_rate < MTK_DSI_DATA_RATE_MIN_MHZ) {
- printk(BIOS_ERR, "data rate (%dMbps) must be >=%dMbps. "
- "Please check the pixel clock (%u), bits per pixel(%u), "
+ u32 data_rate = DIV_ROUND_UP((u64)edid->mode.pixel_clock *
+ bits_per_pixel * 1000 *
+ MTK_DSI_MIPI_RATIO_NUMERATOR,
+ (u64)lanes *
+ MTK_DSI_MIPI_RATIO_DENOMINATOR);
+ printk(BIOS_INFO, "DSI data_rate: %u bps\n", data_rate);
+
+ if (data_rate < MTK_DSI_DATA_RATE_MIN_MHZ * MHz) {
+ printk(BIOS_ERR, "data rate (%ubps) must be >= %ubps. "
+ "Please check the pixel clock (%u), "
+ "bits per pixel (%u), "
"mipi_ratio (%d%%) and number of lanes (%d)\n",
- data_rate, MTK_DSI_DATA_RATE_MIN_MHZ,
+ data_rate, MTK_DSI_DATA_RATE_MIN_MHZ * MHz,
edid->mode.pixel_clock, bits_per_pixel,
(100 * MTK_DSI_MIPI_RATIO_NUMERATOR /
MTK_DSI_MIPI_RATIO_DENOMINATOR), lanes);
- return -1;
+ return 0;
}
return data_rate;
}
@@ -71,12 +74,13 @@ __weak void mtk_dsi_override_phy_timing(struct mtk_phy_timing *timing)
/* Do nothing. */
}
-static void mtk_dsi_phy_timing(int data_rate, struct mtk_phy_timing *phy_timing)
+static void mtk_dsi_phy_timing(u32 data_rate, struct mtk_phy_timing *phy_timing)
{
u32 cycle_time, ui;
+ u32 data_rate_mhz = DIV_ROUND_UP(data_rate, MHz);
- ui = 1000 / data_rate + 0x01;
- cycle_time = 8000 / data_rate + 0x01;
+ ui = 1000 / data_rate_mhz + 0x01;
+ cycle_time = 8000 / data_rate_mhz + 0x01;
memset(phy_timing, 0, sizeof(*phy_timing));
@@ -401,11 +405,11 @@ static void mtk_dsi_reset_dphy(void)
int mtk_dsi_init(u32 mode_flags, u32 format, u32 lanes, const struct edid *edid,
const u8 *init_commands)
{
- int data_rate;
+ u32 data_rate;
u32 bits_per_pixel = mtk_dsi_get_bits_per_pixel(format);
data_rate = mtk_dsi_get_data_rate(bits_per_pixel, lanes, edid);
- if (data_rate < 0)
+ if (!data_rate)
return -1;
mtk_dsi_configure_mipi_tx(data_rate, lanes);
@@ -39,29 +39,32 @@ static unsigned int mtk_dsi_get_bits_per_pixel(u32 format)
{
/* data_rate = pixel_clock * bits_per_pixel * mipi_ratio / lanes
- * Note pixel_clock comes in kHz and returned data_rate is in Mbps.
+ * Note pixel_clock comes in kHz and returned data_rate is in bps.
* mipi_ratio is the clk coefficient to balance the pixel clk in MIPI
* for older platforms which do not have complete implementation in HFP.